==> virtualfs-1.22.src.tar.gz: virtualfs-1.22/tcpaccess.cc <==
#pragma implementation
#include <stdio.h>
#include "tcpaccess.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <syslog.h>
#include "remaudio.h"


PUBLIC TCPACCESS_ITEM::TCPACCESS_ITEM()
{
	ip = 0;
	mask = 0xffffffff;
	next = NULL;
}

PUBLIC TCPACCESS_ITEM::TCPACCESS_ITEM(
	unsigned long _ip,
	unsigned long _mask,
	TCPACCESS_ITEM *_next)
{
	ip = _ip;
	mask = _mask;
	next = _next;
}

PUBLIC TCPACCESS::TCPACCESS()
{
	first = NULL;
}

PUBLIC TCPACCESS::~TCPACCESS()
{
	while (first != NULL){
		TCPACCESS_ITEM *next = first->next;
		delete first;
		first = next;
	}
}

/*
	Return true if a socket is allowed
*/
PUBLIC bool TCPACCESS::isok (int fd)
{
	bool ret = true;
fprintf (stderr,"first = %p\n",first);
	if (first != NULL){
		ret = false;
		struct sockaddr_in adr;
		unsigned int len = sizeof(adr);
		if (getpeername (fd,(struct sockaddr*)&adr,&len) != -1){
			unsigned long addr = ntohl(adr.sin_addr.s_addr);
			TCPACCESS_ITEM *it = first;
			while (it != NULL){
				// fprintf (stderr,"Compare %08lx %08lx %08lx\n",addr,it->ip,it->mask);
				if (it->ip == (addr & it->mask)){
					ret = true;
					break;
				}
				it = it->next;
			}
			if (it == NULL){
				char ipstr[20];
				__ipnum_ip2a (addr,ipstr);
				syslog (LOG_AUTHPRIV,"Reject access from %s",ipstr);
			}
		}else{
			syslog (LOG_ERR,"getpeername (%m)");
		}
	}
	return ret;
}

/*
	Add a host or network/netmask pair which is allowed to connect
*/
PUBLIC void TCPACCESS::add (const char *netdef)
{
	const char *slash = strchr(netdef,'/');
	long ip = 0;
	long mask = 0xffffffffl;
	if (slash != NULL){
		char ipstr[strlen(netdef)];
		char *pt = ipstr;
		while (*netdef != '/'){
			*pt++ = *netdef++;
		}
		*pt = '\0';
		ip = __ipnum_aip2l (ipstr);
		netdef++;
		mask = __ipnum_aip2l (netdef);
	}else{
		ip = __ipnum_aip2l (netdef);
	}
	TCPACCESS_ITEM *it = new TCPACCESS_ITEM (ip,0xffffffffl,first);
	first = it;
}

==> virtualfs-1.24.src.tar.gz: virtualfs-1.24/tcpaccess.cc <==
#pragma implementation
#include <stdio.h>
#include "tcpaccess.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <syslog.h>
#include "remaudio.h"


PUBLIC TCPACCESS_ITEM::TCPACCESS_ITEM()
{
	ip = 0;
	mask = 0xffffffff;
	next = NULL;
}

PUBLIC TCPACCESS_ITEM::TCPACCESS_ITEM(
	unsigned long _ip,
	unsigned long _mask,
	TCPACCESS_ITEM *_next)
{
	ip = _ip;
	mask = _mask;
	next = _next;
}

PUBLIC TCPACCESS::TCPACCESS()
{
	first = NULL;
}

PUBLIC TCPACCESS::~TCPACCESS()
{
	while (first != NULL){
		TCPACCESS_ITEM *next = first->next;
		delete first;
		first = next;
	}
}

/*
	Return true if a socket is allowed
*/
PUBLIC bool TCPACCESS::isok (int fd)
{
	bool ret = true;
	// fprintf (stderr,"first = %p\n",first);
	if (first != NULL){
		ret = false;
		struct sockaddr_in adr;
		unsigned int len = sizeof(adr);
		if (getpeername (fd,(struct sockaddr*)&adr,&len) != -1){
			unsigned long addr = ntohl(adr.sin_addr.s_addr);
			TCPACCESS_ITEM *it = first;
			while (it != NULL){
				// fprintf (stderr,"Compare %08lx %08lx %08lx\n",addr,it->ip,it->mask);
				if (it->ip == (addr & it->mask)){
					ret = true;
					break;
				}
				it = it->next;
			}
			if (it == NULL){
				char ipstr[20];
				__ipnum_ip2a (addr,ipstr);
				syslog (LOG_AUTHPRIV,"Reject access from %s",ipstr);
			}
		}else{
			syslog (LOG_ERR,"getpeername (%m)");
		}
	}
	return ret;
}

/*
	Add a host or network/netmask pair which is allowed to connect
*/
PUBLIC void TCPACCESS::add (const char *netdef)
{
	const char *slash = strchr(netdef,'/');
	long ip = 0;
	long mask = 0xffffffffl;
	if (slash != NULL){
		char ipstr[strlen(netdef)];
		char *pt = ipstr;
		while (*netdef != '/'){
			*pt++ = *netdef++;
		}
		*pt = '\0';
		ip = __ipnum_aip2l (ipstr);
		netdef++;
		mask = __ipnum_aip2l (netdef);
	}else{
		ip = __ipnum_aip2l (netdef);
	}
	TCPACCESS_ITEM *it = new TCPACCESS_ITEM (ip,0xffffffffl,first);
	first = it;
}
